import * as React from 'react';
import { Animated, BackHandler, Dimensions, TouchableWithoutFeedback } from 'react-native';
import { fixSides, sidesToMargin, sidesToPadding } from '../util/sides';
const safeAreaGap = 64;
/**
 * A modal that slides a modal up from the bottom of the screen
 * and dims the rest of the app.
 */
export function AirshipModal(props) {
    const { bridge, children, onCancel, backgroundColor = 'white', borderRadius = 10, borderColor, borderWidth = 0, center = false, flexDirection, justifyContent, maxHeight, maxWidth = 512, overflow = 'visible', shadowOffset = { height: 0, width: 0 }, shadowOpacity = 1, shadowRadius = 10, slideInMs = 300, slideOutMs = 300, underlay = 'rgba(0, 0, 0, 0.75)' } = props;
    const margin = sidesToMargin(fixSides(props.margin, 0));
    const padding = sidesToPadding(fixSides(props.padding, 0));
    React.useEffect(() => bridge.on('clear', onCancel), [bridge, onCancel]);
    // Create the animations:
    const offset = React.useRef(new Animated.Value(Dimensions.get('window').height)).current;
    const opacity = React.useRef(new Animated.Value(0)).current;
    React.useEffect(() => {
        // Animate in:
        Animated.parallel([
            Animated.timing(opacity, {
                toValue: 1,
                duration: slideInMs,
                useNativeDriver: true
            }),
            Animated.timing(offset, {
                toValue: 0,
                duration: slideInMs,
                useNativeDriver: true
            })
        ]).start();
        // Animate out:
        bridge.on('result', () => {
            Animated.parallel([
                Animated.timing(opacity, {
                    toValue: 0,
                    duration: slideOutMs,
                    useNativeDriver: true
                }),
                Animated.timing(offset, {
                    toValue: Dimensions.get('window').height,
                    duration: slideOutMs,
                    useNativeDriver: true
                })
            ]).start(bridge.remove);
        });
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    // Set up the back-button handler:
    React.useEffect(() => {
        const backHandler = BackHandler.addEventListener('hardwareBackPress', () => {
            onCancel();
            return true;
        });
        return () => backHandler.remove();
    }, [onCancel]);
    const underlayStyle = {
        backgroundColor: typeof underlay === 'string' ? underlay : 'transparent',
        bottom: 0,
        left: 0,
        opacity: opacity,
        position: 'absolute',
        right: 0,
        top: 0
    };
    const bodyCommon = Object.assign(Object.assign(Object.assign({}, margin), padding), { alignSelf: center ? 'center' : 'flex-end', backgroundColor,
        flexDirection, flexShrink: 1, justifyContent,
        maxHeight,
        overflow,
        shadowOffset,
        shadowOpacity,
        shadowRadius, transform: [{ translateY: offset }], width: maxWidth // This works because flexShrink is set
     });
    const bodyStyle = center
        ? Object.assign(Object.assign({}, bodyCommon), { borderRadius,
            borderColor,
            borderWidth }) : Object.assign(Object.assign({}, bodyCommon), { borderTopLeftRadius: borderRadius, borderTopRightRadius: borderRadius, borderColor,
        borderWidth, marginBottom: -safeAreaGap, paddingBottom: padding.paddingBottom + safeAreaGap });
    return (React.createElement(React.Fragment, null,
        React.createElement(TouchableWithoutFeedback, { onPress: () => onCancel() },
            React.createElement(Animated.View, { style: underlayStyle }, typeof underlay !== 'string' ? underlay : undefined)),
        React.createElement(Animated.View, { style: bodyStyle }, children)));
}
